@@ -8,6 +8,7 @@ import Control.Exception
 import Test.Tasty.Options
 import Test.Tasty.Patterns
 import Data.Foldable
+import qualified Data.Sequence as Seq
 import Data.Monoid
 import Data.Typeable
 import qualified Data.Map as Map
@@ -107,7 +108,10 @@ class Typeable t => IsTest t where
   run
     :: OptionSet -- ^ options
     -> t -- ^ the test to run
-    -> (Progress -> IO ()) -- ^ a callback to report progress
+    -> (Progress -> IO ()) -- ^ a callback to report progress.
+                           -- Note: the callback is a no-op at the moment
+                           -- and there are no plans to use it;
+                           -- feel free to ignore this argument for now.
     -> IO Result
 
   -- | The list of options that affect execution of tests of this type
@@ -223,20 +227,20 @@ foldTestTree
   -> TestTree
      -- ^ the tree to fold
   -> b
-foldTestTree (TreeFold fTest fGroup fResource) opts tree =
-  let pat = lookupOption opts
-  in go pat [] opts tree
+foldTestTree (TreeFold fTest fGroup fResource) opts0 tree0 =
+  let pat = lookupOption opts0
+  in go pat mempty opts0 tree0
   where
-    go pat path opts tree =
-      case tree of
+    go pat path opts tree1 =
+      case tree1 of
         SingleTest name test
-          | testPatternMatches pat (path ++ [name])
+          | testPatternMatches pat (path Seq.|> name)
             -> fTest opts name test
           | otherwise -> mempty
         TestGroup name trees ->
-          fGroup name $ foldMap (go pat (path ++ [name]) opts) trees
+          fGroup name $ foldMap (go pat (path Seq.|> name) opts) trees
         PlusTestOptions f tree -> go pat path (f opts) tree
-        WithResource res tree -> fResource res $ \res -> go pat path opts (tree res)
+        WithResource res0 tree -> fResource res0 $ \res -> go pat path opts (tree res)
         AskOptions f -> go pat path opts (f opts)
 
 -- | Get the list of options that are relevant for a given test tree
