@@ -8,6 +8,7 @@ module Test.Tasty.Ingredients
   , ingredientOptions
   , ingredientsOptions
   , suiteOptions
+  , composeReporters
   ) where
 
 import Control.Monad
@@ -18,6 +19,7 @@ import Test.Tasty.Core
 import Test.Tasty.Run
 import Test.Tasty.Options
 import Test.Tasty.Options.Core
+import Control.Concurrent.Async (concurrently)
 
 -- | 'Ingredient's make your test suite tasty.
 --
@@ -45,15 +47,20 @@ import Test.Tasty.Options.Core
 -- That's all you need to know from an (advanced) user perspective. Read
 -- on if you want to create a new ingredient.
 --
--- There are two kinds of ingredients. 'TestReporter', if it agrees to run,
--- automatically launches tests execution, and gets the 'StatusMap' which
--- it uses to report the progress and results to the user.
+-- There are two kinds of ingredients.
+--
+-- The first kind is 'TestReporter'. If the ingredient that agrees to run
+-- is a 'TestReporter', then tasty will automatically launch the tests and
+-- pass a 'StatusMap' to the ingredient. All the ingredient needs to do
+-- then is to process the test results and probably report them to the user
+-- in some way (hence the name).
 --
 -- 'TestManager' is the second kind of ingredient. It is typically used for
 -- test management purposes (such as listing the test names), although it
 -- can also be used for running tests (but, unlike 'TestReporter', it has
--- to launch the tests manually).  It is therefore more general than
--- 'TestReporter'. 'TestReporter' is provided just for convenience.
+-- to launch the tests manually if it wants them to be run).  It is
+-- therefore more general than 'TestReporter'. 'TestReporter' is provided
+-- just for convenience.
 --
 -- The function's result should indicate whether all the tests passed.
 --
@@ -64,7 +71,9 @@ import Test.Tasty.Options.Core
 data Ingredient
   = TestReporter
       [OptionDescription]
-      (OptionSet -> TestTree -> Maybe (StatusMap -> IO Bool))
+      (OptionSet -> TestTree -> Maybe (StatusMap -> IO (Time -> IO Bool)))
+   -- ^ For the explanation on how the callback works, see the
+   -- documentation for 'launchTestTree'.
   | TestManager
       [OptionDescription]
       (OptionSet -> TestTree -> Maybe (IO Bool))
@@ -112,3 +121,21 @@ suiteOptions ins tree =
   coreOptions ++
   ingredientsOptions ins ++
   treeOptions tree
+
+-- | Compose two 'TestReporter' ingredients which are then executed
+-- in parallel. This can be useful if you want to have two reporters
+-- active at the same time, e.g., one which prints to the console and
+-- one which writes the test results to a file.
+--
+-- Be aware that it is not possible to use 'composeReporters' with a 'TestManager',
+-- it only works for 'TestReporter' ingredients.
+composeReporters :: Ingredient -> Ingredient -> Ingredient
+composeReporters (TestReporter o1 f1) (TestReporter o2 f2) =
+  TestReporter (o1 ++ o2) $ \o t ->
+  case (f1 o t, f2 o t) of
+    (g, Nothing) -> g
+    (Nothing, g) -> g
+    (Just g1, Just g2) -> Just $ \s -> do
+      (h1, h2) <- concurrently (g1 s) (g2 s)
+      return $ \x -> fmap (uncurry (&&)) $ concurrently (h1 x) (h2 x)
+composeReporters _ _ = error "Only TestReporters can be composed"
