@@ -37,8 +37,11 @@ import Test.Tasty.Runners.Utils
 import Text.Printf
 import qualified Data.IntMap as IntMap
 import Data.Char
+#ifdef UNIX
+import Data.Char.WCWidth (wcwidth)
+#endif
 import Data.Maybe
-import Data.Monoid
+import Data.Monoid (Any(..))
 import Data.Typeable
 import Options.Applicative hiding (str)
 import System.IO
@@ -50,6 +53,8 @@ import Data.Foldable hiding (concatMap,elem,sequence_)
 #if MIN_VERSION_base(4,9,0)
 import Data.Semigroup (Semigroup)
 import qualified Data.Semigroup (Semigroup((<>)))
+#else
+import Data.Monoid
 #endif
 
 --------------------------------------------------
@@ -105,7 +110,7 @@ buildTestOutput opts tree =
       let
         printTestName = do
           printf "%s%s: %s" (indent level) name
-            (replicate (alignment - indentSize * level - length name) ' ')
+            (replicate (alignment - indentSize * level - stringWidth name) ' ')
           hFlush stdout
 
         printTestResult result = do
@@ -539,13 +544,13 @@ instance Ord a => Semigroup (Maximum a) where
   (<>) = mappend
 #endif
 
--- | Compute the amount of space needed to align "OK"s and "FAIL"s
+-- | Compute the amount of space needed to align \"OK\"s and \"FAIL\"s
 computeAlignment :: OptionSet -> TestTree -> Int
 computeAlignment opts =
   fromMonoid .
   foldTestTree
     trivialFold
-      { foldSingle = \_ name _ level -> Maximum (length name + level)
+      { foldSingle = \_ name _ level -> Maximum (stringWidth name + level)
       , foldGroup = \_ m -> m . (+ indentSize)
       }
     opts
@@ -555,6 +560,23 @@ computeAlignment opts =
         MinusInfinity -> 0
         Maximum x -> x
 
+-- | Compute the length/width of the string as it would appear in a monospace
+--   terminal. This takes into account that even in a “mono”space font, not
+--   all characters actually have the same width, in particular, most CJK
+--   characters have twice the same as Western characters.
+--
+--   (This only works properly on Unix at the moment; on Windows, the function
+--   treats every character as width-1 like 'Data.List.length' does.)
+stringWidth :: String -> Int
+#ifdef UNIX
+stringWidth = Prelude.sum . map charWidth
+ where charWidth c = case wcwidth c of
+        -1 -> 1  -- many chars have "undefined" width; default to 1 for these.
+        w  -> w
+#else
+stringWidth = length
+#endif
+
 -- (Potentially) colorful output
 ok, fail, infoOk, infoFail :: (?colors :: Bool) => String -> IO ()
 fail     = output BoldIntensity   Vivid Red
