@@ -1,5 +1,5 @@
 -- | Core options, i.e. the options used by tasty itself
-{-# LANGUAGE DeriveDataTypeable, GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE CPP, DeriveDataTypeable, GeneralizedNewtypeDeriving #-}
 {-# OPTIONS_GHC -fno-warn-type-defaults #-} -- for (^)
 module Test.Tasty.Options.Core
   ( NumThreads(..)
@@ -12,12 +12,10 @@ module Test.Tasty.Options.Core
 import Control.Monad (mfilter)
 import Data.Proxy
 import Data.Typeable
-import Data.Tagged
 import Data.Fixed
 import Data.Monoid
-import Options.Applicative
+import Options.Applicative hiding (str)
 import GHC.Conc
-import Prelude  -- Silence FTP import warnings
 
 import Test.Tasty.Options
 import Test.Tasty.Patterns
@@ -36,7 +34,7 @@ instance IsOption NumThreads where
   parseValue = mfilter onlyPositive . fmap NumThreads . safeRead
   optionName = return "num-threads"
   optionHelp = return "Number of threads to use for tests execution"
-  optionCLParser = mkOptionCLParser (short 'j')
+  optionCLParser = mkOptionCLParser (short 'j' <> metavar "NUMBER")
 
 -- | Filtering function to prevent non-positive number of threads
 onlyPositive :: NumThreads -> Bool
@@ -59,7 +57,7 @@ instance IsOption Timeout where
       <*> pure str
   optionName = return "timeout"
   optionHelp = return "Timeout for individual tests (suffixes: ms,s,m,h; default: s)"
-  optionCLParser = mkOptionCLParser (short 't')
+  optionCLParser = mkOptionCLParser (short 't' <> metavar "DURATION")
 
 parseTimeout :: String -> Maybe Integer
 parseTimeout str =
