@@ -1,6 +1,6 @@
 -- | Running tests
 {-# LANGUAGE ScopedTypeVariables, ExistentialQuantification, RankNTypes,
-             FlexibleContexts, BangPatterns #-}
+             FlexibleContexts, BangPatterns, CPP #-}
 module Test.Tasty.Run
   ( Status(..)
   , StatusMap
@@ -11,9 +11,13 @@ import qualified Data.IntMap as IntMap
 import qualified Data.Sequence as Seq
 import qualified Data.Foldable as F
 import Data.Maybe
+#ifndef VERSION_clock
+import Data.Time.Clock.POSIX (getPOSIXTime)
+#endif
 import Control.Monad.State
 import Control.Monad.Writer
 import Control.Monad.Reader
+import Control.Concurrent
 import Control.Concurrent.STM
 import Control.Concurrent.Timeout (timeout)
 import Control.Concurrent.Async
@@ -22,7 +26,9 @@ import Control.Applicative
 import Control.Arrow
 import GHC.Conc (labelThread)
 import Prelude  -- Silence AMP and FTP import warnings
+#ifdef VERSION_clock
 import qualified System.Clock as Clock
+#endif
 
 import Test.Tasty.Core
 import Test.Tasty.Parallel
@@ -50,6 +56,7 @@ data Resource r
   | BeingCreated
   | FailedToCreate SomeException
   | Created r
+  | BeingDestroyed
   | Destroyed
 
 instance Show (Resource r) where
@@ -58,10 +65,9 @@ instance Show (Resource r) where
     BeingCreated -> "BeingCreated"
     FailedToCreate exn -> "FailedToCreate " ++ show exn
     Created {} -> "Created"
+    BeingDestroyed -> "BeingDestroyed"
     Destroyed -> "Destroyed"
 
-data ResourceVar = forall r . ResourceVar (TVar (Resource r))
-
 data Initializer
   = forall res . Initializer
       (IO res)
@@ -129,8 +135,15 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
             BeingCreated -> retry
             Created {} -> return $ return ()
             FailedToCreate exn -> return $ throwIO exn
-            _ -> return $ throwIO $
-              unexpectedState "initResources" resStatus
+            -- If the resource is destroyed or being destroyed
+            -- while we're starting a test, the test suite is probably
+            -- shutting down. We are about to be killed.
+            -- (In fact we are probably killed already, so these cases are
+            -- unlikely to occur.)
+            -- In any case, the most sensible thing to do is to go to
+            -- sleep, awaiting our fate.
+            Destroyed      -> return $ sleepIndefinitely
+            BeingDestroyed -> return $ sleepIndefinitely
 
     applyTimeout :: Timeout -> IO Result -> IO Result
     applyTimeout NoTimeout a = a
@@ -156,7 +169,7 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
     destroyResources restore = do
       -- remember the first exception that occurred
       liftM getFirst . execWriterT . getTraversal $
-        flip F.foldMap fins $ \(Finalizer doRelease initVar finishVar) ->
+        flip F.foldMap fins $ \fin@(Finalizer _ _ finishVar) ->
           Traversal $ do
             iAmLast <- liftIO $ atomically $ do
               nUsers <- readTVar finishVar
@@ -166,19 +179,7 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
 
             mbExcn <- liftIO $
               if iAmLast
-              then join $ atomically $ do
-                resStatus <- readTVar initVar
-                case resStatus of
-                  Created res -> do
-                    -- Don't worry about double destroy — only one thread
-                    -- receives iAmLast
-                    return $
-                      (either Just (const Nothing)
-                        <$> try (restore $ doRelease res))
-                        <* atomically (writeTVar initVar Destroyed)
-                  FailedToCreate {} -> return $ return Nothing
-                  _ -> return $ return $ Just $
-                    unexpectedState "destroyResources" resStatus
+              then destroyResource restore fin
               else return Nothing
 
             tell $ First mbExcn
@@ -194,19 +195,22 @@ executeTest action statusVar timeoutOpt inits fins = mask $ \restore -> do
 type InitFinPair = (Seq.Seq Initializer, Seq.Seq Finalizer)
 
 -- | Turn a test tree into a list of actions to run tests coupled with
--- variables to watch them
-createTestActions :: OptionSet -> TestTree -> IO ([(IO (), TVar Status)], [ResourceVar])
-createTestActions opts tree = do
+-- variables to watch them.
+--
+-- Also return a list of finalizers in the order they should run when the
+-- test suite completes.
+createTestActions :: OptionSet -> TestTree -> IO ([(IO (), TVar Status)], Seq.Seq Finalizer)
+createTestActions opts0 tree = do
   let
     traversal ::
-      Traversal (WriterT ([(InitFinPair -> IO (), TVar Status)], [ResourceVar]) IO)
+      Traversal (WriterT ([(InitFinPair -> IO (), TVar Status)], Seq.Seq Finalizer) IO)
     traversal =
       foldTestTree
         trivialFold
           { foldSingle = runSingleTest
           , foldResource = addInitAndRelease
           }
-        opts tree
+        opts0 tree
   (tests, rvars) <- unwrap traversal
   let tests' = map (first ($ (Seq.empty, Seq.empty))) tests
   return (tests', rvars)
@@ -220,14 +224,14 @@ createTestActions opts tree = do
       tell ([(act, statusVar)], mempty)
     addInitAndRelease (ResourceSpec doInit doRelease) a = wrap $ do
       initVar <- atomically $ newTVar NotCreated
-      (tests, rvars) <- unwrap $ a (getResource initVar)
+      (tests, fins) <- unwrap $ a (getResource initVar)
       let ntests = length tests
       finishVar <- atomically $ newTVar ntests
       let
         ini = Initializer doInit initVar
         fin = Finalizer doRelease initVar finishVar
         tests' = map (first $ local $ (Seq.|> ini) *** (fin Seq.<|)) tests
-      return (tests', ResourceVar initVar : rvars)
+      return (tests', fins Seq.|> fin)
     wrap = Traversal . WriterT . fmap ((,) ())
     unwrap = execWriterT . getTraversal
 
@@ -241,9 +245,47 @@ getResource var =
       Destroyed -> throwSTM UseOutsideOfTest
       _ -> throwSTM $ unexpectedState "getResource" rState
 
--- | Start running all the tests in a test tree in parallel, without
--- blocking the current thread. The number of test running threads is
--- determined by the 'NumThreads' option.
+-- | Run a resource finalizer.
+--
+-- This function is called from two different places:
+--
+-- 1. A test thread, which is the last one to use the resource.
+-- 2. The main thread, if an exception (e.g. Ctrl-C) is received.
+--
+-- Therefore, it is possible that this function is called multiple
+-- times concurrently on the same finalizer.
+--
+-- This function should be run with async exceptions masked,
+-- and the restore function should be passed as an argument.
+destroyResource :: (forall a . IO a -> IO a) -> Finalizer -> IO (Maybe SomeException)
+destroyResource restore (Finalizer doRelease stateVar _) = join . atomically $ do
+  rState <- readTVar stateVar
+  case rState of
+    Created res -> do
+      writeTVar stateVar BeingDestroyed
+      return $
+        (either Just (const Nothing)
+          <$> try (restore $ doRelease res))
+          <* atomically (writeTVar stateVar Destroyed)
+    BeingCreated   -> retry
+    -- If the resource is being destroyed, wait until it is destroyed.
+    -- This is so that we don't start destroying the next resource out of
+    -- order.
+    BeingDestroyed -> retry
+    NotCreated -> do
+      -- prevent the resource from being created by a competing thread
+      writeTVar stateVar Destroyed
+      return $ return Nothing
+    FailedToCreate {} -> return $ return Nothing
+    Destroyed         -> return $ return Nothing
+
+-- | Start running the tests (in background, in parallel) and pass control
+-- to the callback.
+--
+-- Once the callback returns, stop running the tests.
+--
+-- The number of test running threads is determined by the 'NumThreads'
+-- option.
 launchTestTree
   :: OptionSet
   -> TestTree
@@ -261,17 +303,24 @@ launchTestTree
     -- all resource initializers and finalizers, which is why it is more
     -- accurate than what could be measured from inside the first callback.
   -> IO a
-launchTestTree opts tree k = do
-  (testActions, rvars) <- createTestActions opts tree
+launchTestTree opts tree k0 = do
+  (testActions, fins) <- createTestActions opts tree
   let NumThreads numTheads = lookupOption opts
-  (t,k) <- timed $ do
+  (t,k1) <- timed $ do
      abortTests <- runInParallel numTheads (fst <$> testActions)
      (do let smap = IntMap.fromList $ zip [0..] (snd <$> testActions)
-         k smap)
-      `finally` do
+         k0 smap)
+      `finallyRestore` \restore -> do
+         -- Tell all running tests to wrap up.
          abortTests
-         waitForResources rvars
-  k t
+         -- Destroy all allocated resources in the case they didn't get
+         -- destroyed by their tests. (See #75.)
+         F.mapM_ (destroyResource restore) fins
+         -- Wait until all resources are destroyed. (Specifically, those
+         -- that were being destroyed by their tests, not those that were
+         -- destroyed by destroyResource above.)
+         restore $ waitForResources fins
+  k1 t
   where
     alive :: Resource r -> Bool
     alive r = case r of
@@ -279,16 +328,35 @@ launchTestTree opts tree k = do
       BeingCreated -> True
       FailedToCreate {} -> False
       Created {} -> True
+      BeingDestroyed -> True
       Destroyed -> False
 
-    waitForResources rvars = atomically $
-      forM_ rvars $ \(ResourceVar rvar) -> do
+    waitForResources fins = atomically $
+      F.forM_ fins $ \(Finalizer _ rvar _) -> do
         res <- readTVar rvar
         check $ not $ alive res
 
 unexpectedState :: String -> Resource r -> SomeException
 unexpectedState where_ r = toException $ UnexpectedState where_ (show r)
 
+sleepIndefinitely :: IO ()
+sleepIndefinitely = forever $ threadDelay (10^(7::Int))
+
+-- | Like 'finally' (which also masks its finalizers), but pass the restore
+-- action to the finalizer.
+finallyRestore
+  :: IO a
+    -- ^ computation to run first
+  -> ((forall c . IO c -> IO c) -> IO b)
+    -- ^ computation to run afterward (even if an exception was raised)
+  -> IO a
+    -- ^ returns the value from the first computation
+a `finallyRestore` sequel =
+  mask $ \restore -> do
+    r <- restore a `onException` sequel restore
+    _ <- sequel restore
+    return r
+
 -- | Measure the time taken by an 'IO' action to run
 timed :: IO a -> IO (Time, a)
 timed t = do
@@ -297,6 +365,7 @@ timed t = do
   end   <- getTime
   return (end-start, r)
 
+#ifdef VERSION_clock
 -- | Get monotonic time
 --
 -- Warning: This is not the system time, but a monotonically increasing time
@@ -304,5 +373,15 @@ timed t = do
 getTime :: IO Time
 getTime = do
   t <- Clock.getTime Clock.Monotonic
-  let ns = realToFrac $ Clock.timeSpecAsNanoSecs t
+  let ns = realToFrac $
+#if MIN_VERSION_clock(0,7,1)
+        Clock.toNanoSecs t
+#else
+        Clock.timeSpecAsNanoSecs t
+#endif
   return $ ns / 10 ^ (9 :: Int)
+#else
+-- | Get system time
+getTime :: IO Time
+getTime = realToFrac <$> getPOSIXTime
+#endif
